/* $Id: dijkstra.c,v 1.3 2005/05/02 17:56:39 dw Exp $ */

/* implementation of Dijkstra's algorithm */

/* Copyright (C) 2003 Damian Weber dweber@htw-saarland.de

   This file is part of the Damian Weber's Dijkstra implementation, 
   a collection of source files to compute shortest paths in graphs
   with non-negative weights.

   Damian Weber's Dijkstra implementation is free software; you can 
   redistribute it and/or modify it under the terms of the GNU General 
   Public License as published by the Free Software Foundation; either 
   version 2, or (at your option) any later version.

   Damian Weber's Dijkstra implementation is distributed in the hope 
   that it will be useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
   See the GNU General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. 
*/

#include <malloc.h>
#include <stdio.h>

#include "graph_functions.h"

/* standard relax */
static void relax(int **w, int n, int *d, int u, int v, int *pred)
{
	int d1;

	d1=d[u]+w[u][v];
	if (d[v]>d1)
	{
		d[v]=d1;
		pred[v]=u;
	}
}

/* find minimum distance in array d[]    */
/* this ought to be done by using a heap */
int find_min_distance(int n,int *d,int *visited)
{
	int min, min_u, u;

	min_u=-1;
	min=INFTY;
	for (u=0;u<n;u++)
		if (!visited[u] && d[u]<min)
		{
			min=d[u]; min_u=u;
		}
	return min_u;
}

/* output of distances to stdout */
static void output_d(int n,int *d, int *pred, int u)
{
	int v;

	printf("vertex %d : ",u+1); 
	for (v=0;v<n;v++)
		if (d[v]==INFTY) printf(" -");
		else printf(" %d",d[v]);
		printf("    pred: ");
		for (v=0;v<n;v++)
			if (pred[v]==-1) printf(" -");
			else printf(" %d",pred[v]+1);
			printf("\n");
}

/* Dijkstra's algorithm */
void dijkstra(int **w, int n, int *d, int s, int *pred)
{
	int i;
	int u,v;
	int *visited;

	visited=malloc(n*sizeof(int));
	for (i=0;i<n;i++)
	{
		d[i]=INFTY;
		pred[i]=-1;
		visited[i]=0;
	}

	d[s]=0;

#if 0
	output_d(n,d,pred,-1);
#endif

	while ((u=find_min_distance(n,d,visited))>=0)
	{
		visited[u]=1;
		for (v=0;v<n;v++)
			if (w[u][v]<INFTY) 
				relax(w,n,d,u,v,pred);
/*		output_d(n,d,pred,u); */
	}
	free(visited);
}

/* print recursively a shortest path from s to v */
void print_shortest_path(int s,int v,int *pred)
{

	if (s==v) printf(" %d",s+1);
	else 
	{
		if (pred[v]<0)
		{
			printf("%d not reachable\n",v+1);
			return;
		}
		print_shortest_path(s,pred[v],pred);
		printf(" %d",v+1);
	}
}

/* print shortest paths from s to all vertices */
void print_shortest_paths(int n,int s,int *pred)
{
	int v;

	for (v=0;v<n;v++)
	{
		print_shortest_path(s,v,pred);
		printf("\n");
	}
}
