/* test program for the Dijkstra implementation
 *
 * $Id: dijkstra_run.c,v 1.2 2003/06/09 12:01:22 dw Exp $
 */

/* Copyright (C) 2003 Damian Weber dweber@htw-saarland.de

   This file is part of the Damian Weber's Dijkstra implementation, 
   a collection of source files to compute shortest paths in graphs
   with non-negative weights.

   Damian Weber's Dijkstra implementation is free software; you can 
   redistribute it and/or modify it under the terms of the GNU General 
   Public License as published by the Free Software Foundation; either 
   version 2, or (at your option) any later version.

   Damian Weber's Dijkstra implementation is distributed in the hope 
   that it will be useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
   See the GNU General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. 
*/

#include <stdio.h>
#include <stdlib.h>
#include <values.h>
#include <ctype.h>

#include "graph_functions.h"
#include "dijkstra.h"

/* usage: dijkstra_run #nodes|filename */

int main(int argc, char **argv)
{
	int n;
	int i; /* malloc'ing and freeing memory */
	int **w; /* array containing the input weights */
	int *d;  /* array containing the distances from source vertex */
	int *pred;  /* array containing the predecessors on shortest path */
	int rc; /* return code */

	rc=0;
	if (argc<=1) return 1;

	if (isdigit(argv[1][0]))
	{
		sscanf(argv[1],"%d",&n);

		w=(int **) malloc(n*sizeof(int *));
		for (i=0;i<n;i++) w[i]=(int *) malloc(n*sizeof(int ));

		graph_generate(n,w);
	}
	else
		if (!(w=graph_read(argv[1],&n))) return 1;

	d=(int *) malloc(n*sizeof(int));
	pred=(int *) malloc(n*sizeof(int));
	if (!d || !pred)
	{
		printf("memory allocation d, pred failed\n");
		return 1;
	}


	/* Dijkstra */
	dijkstra(w,n,d,0,pred);
	print_shortest_paths(n,0,pred);
	printf("-----------------------------------------\n");

	for (i=0;i<n;i++) free(w[i]);
	free(w);

	free(d);
	free(pred);

	return rc;
}

