/* $Id: graph_functions.c,v 1.2 2003/06/09 12:01:22 dw Exp $ */

/* Copyright (C) 2003 Damian Weber dweber@htw-saarland.de

   This file is part of the Damian Weber's Dijkstra implementation, 
   a collection of source files to compute shortest paths in graphs
   with non-negative weights.

   Damian Weber's Dijkstra implementation is free software; you can 
   redistribute it and/or modify it under the terms of the GNU General 
   Public License as published by the Free Software Foundation; either 
   version 2, or (at your option) any later version.

   Damian Weber's Dijkstra implementation is distributed in the hope 
   that it will be useful, but WITHOUT ANY WARRANTY; without even the 
   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
   See the GNU General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. 
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <values.h>

#include "graph_functions.h"


/* fills adj-matrix with random positive int's below 1000 */
/* 1/8 pf entries are infinity                            */
void graph_generate(int n,int **a)
{
	int i,j;

#if 1
	srand(time(NULL));
#else
	srand(n);
#endif
	for (i=0;i<n;i++)
		for (j=0;j<n;j++)
		{
			if (i==j) a[i][j]=0;
			else
				switch(rand()&7)
				{
					case 0: a[i][j]=INFTY;
						break;
					default: a[i][j]=1+(int) (1000.0*rand()/(RAND_MAX+1.0));
						 break;
				}
		}
}

/* read a graph from file, see file format from graph.in */
int **graph_read(char *fname, int *np)
{
	int i,j;
	int **a;
	FILE *f;
	char buffer[80];

	f=fopen(fname,"r");
	if (!f) return 0;

	if (fscanf(f,"%d",np)<1) return 0;
	a=malloc(*np*sizeof(int *));
	if (!a) return 0;

	for (i=0;i<*np;i++)
	{
		a[i]=malloc(*np*sizeof(int));
		if (!a[i]) return 0;
		for (j=0;j<*np;j++)
		{
			if (fscanf(f,"%s",buffer)<1) return 0;
			if (!strcmp(buffer,"-")) a[i][j]=INFTY;
			else 
				if (!sscanf(buffer,"%d",&a[i][j])) return 0;
		}
	}
	fclose(f);

	return a;
}
